package it.unige.disi.aumlbpel.transformers;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.w3c.dom.Document;

// For write operation
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import java.io.*;

 
/**
 * @author Giovanni Casella
 * it.unige.disi.aumlbpel.transformers
 * XMI_IntermediateXML.java 
 * Questa Classe permette di trasformare
 * un documento xmi rappresentante un Agent Interaction Protocol
 * AUML realizzato seguendo alcune convenzione 
 * in un documento IntermediateXML processabile.
 * La trasformazione avviene applicando un file xsl al documento xmi
 * Il documento xsl applicato e' visionabile nella cartellla 
 * resource/transformerStylesheet
 */
public class XMI_IntermediateXML {
    /**
     * Trasforma 
     * un documento XMI in un documento nel formato IntermediateXML.
     * @param nomefileXmlInput XMI in input
     * @param nomeFileOutput xml in output
     */
        
    public void tranform(String nomefileXmiInput,
            			 String nomeFileXmlOutput)
    {
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        String nomeFileStylesheet="resource\\transformer_stylesheet\\XMI-IntermediateNotationWithContent.xsl";
        
        try {
            Document document;
          
            File stylesheet = new File(nomeFileStylesheet);
            File datafile = new File(nomefileXmiInput);
            File outputfile = new File(nomeFileXmlOutput);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(datafile);
            DOMSource source = new DOMSource(document);
            
            TransformerFactory tF = 
                TransformerFactory.newInstance();
            Transformer tr = tF.newTransformer
            (new javax.xml.transform.stream.StreamSource
                    (stylesheet));
            tr.transform(new javax.xml.transform.stream.StreamSource(datafile),
                    new javax.xml.transform.stream.StreamResult(new FileWriter(outputfile,false)));
            
        } catch (TransformerConfigurationException tce) {
           Throwable x = tce;
            
            if (tce.getException() != null) {
                x = tce.getException();
            }
            
            x.printStackTrace();
        } catch (TransformerException te) {
            
            // Use the contained exception, if any
            Throwable x = te;
            
            if (te.getException() != null) {
                x = te.getException();
            }
            
            x.printStackTrace();
        } catch (SAXException sxe) {
            // Error generated by this application
            // (or a parser-initialization error)
            Exception x = sxe;
            
            if (sxe.getException() != null) {
                x = sxe.getException();
            }
            
            x.printStackTrace();
        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
        }
        
        
    }
    
    /**
     * Main
     * @param argv
     */
    
    public static void main(String[] argv) {
        
        //Input Files
        String nomefileXmlInput= "E:\\EclipseProject\\FromAUMLToBPELExtended\\resource\\testingFile\\fruitSeller.xmi";
        String nomeFileOutput= "E:\\EclipseProject\\FromAUMLToBPELExtended\\resource\\testingFile\\fruitSeller.xml";
        
        XMI_IntermediateXML trans=new XMI_IntermediateXML();
        trans.tranform(nomefileXmlInput,nomeFileOutput);
    }
    
}
