<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:UML="org.omg.xmi.namespace.UML" xmlns:UML2="org.omg.xmi.namespace.UML2">
	<xsl:output method="xml" indent="yes"/>
	<!-- ******************************************* DEFINITION of  VARIABLES *************************************************-->
	<!-- The "label" variable is used to remember the nesting level inside combined fragment-->
	<xsl:variable name="level">1</xsl:variable>
	<!-- The "publisher-lifeline" variable is initialised with the identifier of the publisher's lifeline -->
	<xsl:variable name="publisher_lifeline">
		<xsl:for-each select="//UML2:Interaction.lifeline/UML2:Lifeline">
			<!-- CONSTRAINT: We assume that there are only two agents in the diagram, the reader and the publisher -->
			<!-- CONSTRAINT: We assume that the leftmost lifeline is the one of the publisher agent -->
			<xsl:if test="position()=1">
				<xsl:value-of select="@xmi.id"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<!-- The "reader-lifeline" variable is initialised with the identifier of the reader's lifeline -->
	<xsl:variable name="reader_lifeline">
		<xsl:for-each select="//UML2:Interaction.lifeline/UML2:Lifeline">
			<!--CONSTRAINT: We assume that the rightmost lifeline is the one of the reader agent -->
			<xsl:if test="position()=2">
				<xsl:value-of select="@xmi.id"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<xsl:template match="XMI">
		<protocol-diagram-intermediate-notation>
			<!--*******************************************EXTRACTION OF THE PARAMETERS *******************************************-->
			<!-- The comments named parameters and "optional-param" are extracted from the XMI source -->
			<xsl:for-each select="//UML:Comment[@name='parameters']">
				<parameters>
					<xsl:value-of select="@body"/>
				</parameters>
			</xsl:for-each>
			<xsl:for-each select="//UML:Comment[@name='optional_parameters']">
				<optional-parameters>
					<xsl:value-of select="@body"/>
				</optional-parameters>
			</xsl:for-each>
			<!--The protocol name is extracted from the XMI source -->
			<xsl:for-each select="XMI.content/UML:Diagram">
				<protocol-name>
					<xsl:value-of select="@name"/>
				</protocol-name>
			</xsl:for-each>
			<!-- *********************************** EXTRACTION OF THE AGENTS' FEATURES **********************************************-->
			<!-- The agents' features and the lifeline identifiers are extracted from the XMI source -->
			<xsl:for-each select="//UML2:Interaction.lifeline/UML2:Lifeline">
				<!-- CONSTRAINT: We assume that the leftmost lifeline is the one of the publisher agent, and the rightmost one is that
				of the reader agent -->
				<xsl:if test="position()=1">
					<agent-publisher short-name="p">
						<xsl:value-of select="@name"/>
					</agent-publisher>
				</xsl:if>
				<xsl:if test="position()=2">
					<agent-reader short-name="r">
						<xsl:value-of select="@name"/>
					</agent-reader>
				</xsl:if>
			</xsl:for-each>
			<!--The external interaction fragment, containing all the other interaction fragments and corresponding messages, 
			    is extracted from the XMI source -->
			<xsl:for-each select="//UML2:Interaction/UML2:Interaction.fragment">
				<main-interaction-fragment>
					<xsl:for-each select="./*">
						<!-- A check on the selected child is performed; according to what it is (either a combined fragment or an                               
                event occurrence), the corresponding function is called -->
						<xsl:if test="name()='UML2:CombinedFragment'">
							<xsl:call-template name="manage_combined_fragment">
								<xsl:with-param name="level" select="$level"/>
								<xsl:with-param name="publisher_lifeline" select="$publisher_lifeline"/>
								<xsl:with-param name="reader_lifeline" select="$reader_lifeline"/>
							</xsl:call-template>
						</xsl:if>
						<xsl:if test="name()='UML2:EventOccurrence'">
							<xsl:call-template name="manage_event_occurence">
								<xsl:with-param name="level" select="$level"/>
								<xsl:with-param name="publisher_lifeline" select="$publisher_lifeline"/>
								<xsl:with-param name="reader_lifeline" select="$reader_lifeline"/>
							</xsl:call-template>
						</xsl:if>
						<!--xsl:copy-of select="name()"></xsl:copy-of-->
					</xsl:for-each>
				</main-interaction-fragment>
			</xsl:for-each>
		</protocol-diagram-intermediate-notation>
	</xsl:template>
	<!-- ********************************* DEFINITION of SPECIALISED TEMPLATES *****************************************************-->
	<!--This template is used to identify sent messages -->
	<xsl:template name="manage_event_occurence">
		<xsl:param name="level"/>
		<xsl:param name="publisher_lifeline"/>
		<xsl:param name="reader_lifeline"/>
		<xsl:for-each select="UML2:MessageEnd.sendMessage/UML2:Message">
			<message>
				<sender>
					<xsl:variable name="lifeline_id">
						<xsl:for-each select="../../UML2:InteractionFragment.covered/UML2:Lifeline">
							<xsl:value-of select="@xmi.idref"/>
						</xsl:for-each>
					</xsl:variable>
					<xsl:call-template name="check_message_sender">
						<xsl:with-param name="lifeline_id" select="$lifeline_id"/>
						<xsl:with-param name="publisher_lifeline" select="$publisher_lifeline"/>
						<xsl:with-param name="reader_lifeline" select="$reader_lifeline"/>
					</xsl:call-template>
				</sender>
				<xsl:call-template name="check_message_ca">
					<xsl:with-param name="message_id">
						<xsl:value-of select="@xmi.idref"/>
					</xsl:with-param>
				</xsl:call-template>
				<xsl:call-template name="check_message_content">
					<xsl:with-param name="message_id">
						<xsl:value-of select="@xmi.idref"/>
					</xsl:with-param>
				</xsl:call-template>
			</message>
		</xsl:for-each>
	</xsl:template>
	<!-- This template looks for the message content that we assume to be encoded by means of the "documentation" attribute of the message and written in plain text (use Poseidon source view in Poseidon documentation editor)-->
	<xsl:template name="check_message_content">
		<xsl:param name="message_id"/>
		<xsl:for-each select="//UML2:Interaction.message/UML2:Message[@xmi.id=$message_id]">
			<content>
				<xsl:value-of select="UML:ModelElement.taggedValue/UML:TaggedValue/UML:TaggedValue.dataValue"/>
			</content>
		</xsl:for-each>
	</xsl:template>
	<!-- This template looks for the message's performative (or "communicative act") that we assume to be encoded by means of the "call" 
	     attribute of the message-->
	<xsl:template name="check_message_ca">
		<xsl:param name="message_id"/>
		<xsl:for-each select="//UML2:Interaction.message/UML2:Message[@xmi.id=$message_id]">
			<communicative-act>
				<xsl:value-of select="@name"/>
			</communicative-act>
		</xsl:for-each>
	</xsl:template>
	<!-- This templates is used to find out whether the message's sender is the publisher or the reader -->
	<xsl:template name="check_message_sender">
		<xsl:param name="lifeline_id"/>
		<xsl:param name="publisher_lifeline"/>
		<xsl:param name="reader_lifeline"/>
		<xsl:if test="$reader_lifeline=$lifeline_id">reader</xsl:if>
		<xsl:if test="$publisher_lifeline=$lifeline_id">publisher</xsl:if>
	</xsl:template>
	<!-- This template is used to find the combined fragments contained below the element from which the template is called. 
     It defines a function that recursively inspects the nested fragments -->
	<xsl:template name="manage_combined_fragment">
		<xsl:param name="level"/>
		<xsl:param name="publisher_lifeline" select="$publisher_lifeline"/>
		<xsl:param name="reader_lifeline" select="$reader_lifeline"/>
		<combined-fragment>
			<type>
				<xsl:value-of select="@interactionOperator"/>
			</type>
			<guard>
				<xsl:for-each select="UML2:CombinedFragment.operand/UML2:InteractionOperand/UML2:InteractionOperand.guard/UML2:InteractionConstraint">
					<xsl:value-of select="@name"/>
				</xsl:for-each>
			</guard>
			<xsl:for-each select="UML2:CombinedFragment.operand/UML2:InteractionOperand/UML2:InteractionOperand.fragment">
				<fragment>
					<xsl:for-each select="./*">
						<!-- A check on the selected child is performed; according to what it is (either a combined fragment or an
                                   event occurrence), the corresponding function is called -->
						<xsl:if test="name()='UML2:CombinedFragment'">
							<!-- Recursive call -->
							<xsl:call-template name="manage_combined_fragment">
								<xsl:with-param name="level" select="$level+1"/>
								<xsl:with-param name="publisher_lifeline" select="$publisher_lifeline"/>
								<xsl:with-param name="reader_lifeline" select="$reader_lifeline"/>
							</xsl:call-template>
						</xsl:if>
						<xsl:if test="name()='UML2:EventOccurrence'">
							<xsl:call-template name="manage_event_occurence">
								<xsl:with-param name="level" select="$level"/>
								<xsl:with-param name="publisher_lifeline" select="$publisher_lifeline"/>
								<xsl:with-param name="reader_lifeline" select="$reader_lifeline"/>
							</xsl:call-template>
						</xsl:if>
						<!--xsl:copy-of select="name()"></xsl:copy-of-->
					</xsl:for-each>
				</fragment>
			</xsl:for-each>
		</combined-fragment>
	</xsl:template>
</xsl:stylesheet>
